"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Searches an array for the first repeating element and returns it. Time and space complexity = O(n).
 * @param array The array to search.
 * @returns The first repeating element, null if there are no repeating elements.
 */
function arrayFirstRepeatingElement(array) {
    if (!Array.isArray(array)) {
        throw new TypeError("expected an array but got '" + typeof array + "'");
    }
    if (array.length < 2) {
        return null;
    }
    var valueToIndexMap = new Map([[array[0], 0]]);
    for (var i = 1; i < array.length; ++i) {
        var val = array[i];
        if (valueToIndexMap.get(val) !== undefined) {
            return val;
        }
        valueToIndexMap.set(val, i);
    }
    return null;
}
exports.default = arrayFirstRepeatingElement;
