import { NodeUnilateral as Node } from "../singly_linked_list/SinglyLinkedList";
export { Node };
export default class Queue<T> {
    private _list;
    /**
     * Creates a new instance of a queue (utilizes a singly-linked list).
     * @param values The initial values to populate queue with. If omitted, queue begins empty.
     */
    constructor(values?: T[]);
    /**
     * Adds an item to back of queue. Time complexity = O(1).
     * @param value The item to enqueue.
     * @returns The queue instance - this.
     */
    enqueue(value: T): this;
    /**
     * Removes item at front of queue and returns it. Throws error if queue is empty. Time complexity = O(1).
     * @returns The item at front of queue, or null if queue is empty.
     */
    dequeue(): T | null;
    /**
     * Gets item at specified index. Time complexity = O(n) where n is number of items in queue.
     * @param index The index of item to retrieve.
     * @returns The item at specified index.
     */
    get(index: number): T | null;
    /**
     * @returns An array containing currently held items, ordered from front to back. Time complexity = O(n) where n is number of items in queue.
     */
    getItems(): T[];
    /**
     * Executes a callback function for each item in queue. Time complexity = O(n) where n is number of items in queue.
     * @param callback The function to execute for each item in queue.
     * @returns The queue instance - this.
     */
    forEach(callback: (item: T, index: number) => void): this;
    /**
     * Executes a callback function that replaces each item in queue with result. Time complexity = O(n) where n is number of items in queue.
     * @param callback The function to execute for each item in queue.
     * @returns The queue instance - this.
     */
    forEachMutate(callback: (item: T, index: number) => T): this;
    /**
     * Reverses items in queue: front becomes back and vice versa. Time complexity = O(n) where n is number of items in queue.
     * @returns The queue instance - this.
     */
    reverse(): this;
    /**
     * Removes all items from queue. Time complexity = O(1).
     * @returns The queue instance - this.
     */
    clear(): this;
    /**
     * Time complexity = O(1).
     * @returns True if queue is empty, false otherwise.
     */
    isEmpty(): boolean;
    /**
     * Time complexity = O(1).
     * @returns The current number of items in queue.
     */
    length(): number;
    /**
     * Time complexity = O(1).
     * @returns The item at front of queue, or null if queue is empty.
     */
    first(): T | null;
    /**
     * Time complexity = O(1).
     * @returns The node at front of queue, or null if queue is empty.
     */
    firstNode(): Node<T> | null;
    /**
     * Time complexity = O(1).
     * @returns The item at back of queue, or null if queue is empty.
     */
    last(): T | null;
    /**
     * Time complexity = O(1).
     * @returns The node at back of queue, or null if queue is empty.
     */
    lastNode(): Node<T> | null;
}
