"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Node = void 0;
var SinglyLinkedList_1 = __importStar(require("../singly_linked_list/SinglyLinkedList"));
Object.defineProperty(exports, "Node", { enumerable: true, get: function () { return SinglyLinkedList_1.NodeUnilateral; } });
var linked_list_1 = require("../utilities/linked-list");
var Queue = /** @class */ (function () {
    /**
     * Creates a new instance of a queue (utilizes a singly-linked list).
     * @param values The initial values to populate queue with. If omitted, queue begins empty.
     */
    function Queue(values) {
        this._list = new SinglyLinkedList_1.default(values);
    }
    /**
     * Adds an item to back of queue. Time complexity = O(1).
     * @param value The item to enqueue.
     * @returns The queue instance - this.
     */
    Queue.prototype.enqueue = function (value) {
        this._list.append(value);
        return this;
    };
    /**
     * Removes item at front of queue and returns it. Throws error if queue is empty. Time complexity = O(1).
     * @returns The item at front of queue, or null if queue is empty.
     */
    Queue.prototype.dequeue = function () {
        return this._list.remove(0);
    };
    /**
     * Gets item at specified index. Time complexity = O(n) where n is number of items in queue.
     * @param index The index of item to retrieve.
     * @returns The item at specified index.
     */
    Queue.prototype.get = function (index) {
        return (0, linked_list_1.listGet)(this._list, index);
    };
    /**
     * @returns An array containing currently held items, ordered from front to back. Time complexity = O(n) where n is number of items in queue.
     */
    Queue.prototype.getItems = function () {
        return this._list.getItems();
    };
    /**
     * Executes a callback function for each item in queue. Time complexity = O(n) where n is number of items in queue.
     * @param callback The function to execute for each item in queue.
     * @returns The queue instance - this.
     */
    Queue.prototype.forEach = function (callback) {
        this._list.forEach(callback);
        return this;
    };
    /**
     * Executes a callback function that replaces each item in queue with result. Time complexity = O(n) where n is number of items in queue.
     * @param callback The function to execute for each item in queue.
     * @returns The queue instance - this.
     */
    Queue.prototype.forEachMutate = function (callback) {
        this._list.forEachMutate(callback);
        return this;
    };
    /**
     * Reverses items in queue: front becomes back and vice versa. Time complexity = O(n) where n is number of items in queue.
     * @returns The queue instance - this.
     */
    Queue.prototype.reverse = function () {
        this._list.reverse();
        return this;
    };
    /**
     * Removes all items from queue. Time complexity = O(1).
     * @returns The queue instance - this.
     */
    Queue.prototype.clear = function () {
        this._list.clear();
        return this;
    };
    /**
     * Time complexity = O(1).
     * @returns True if queue is empty, false otherwise.
     */
    Queue.prototype.isEmpty = function () {
        return this._list.isEmpty();
    };
    /**
     * Time complexity = O(1).
     * @returns The current number of items in queue.
     */
    Queue.prototype.length = function () {
        return this._list.length();
    };
    /**
     * Time complexity = O(1).
     * @returns The item at front of queue, or null if queue is empty.
     */
    Queue.prototype.first = function () {
        return this._list.head();
    };
    /**
     * Time complexity = O(1).
     * @returns The node at front of queue, or null if queue is empty.
     */
    Queue.prototype.firstNode = function () {
        return this._list.headNode();
    };
    /**
     * Time complexity = O(1).
     * @returns The item at back of queue, or null if queue is empty.
     */
    Queue.prototype.last = function () {
        return this._list.tail();
    };
    /**
     * Time complexity = O(1).
     * @returns The node at back of queue, or null if queue is empty.
     */
    Queue.prototype.lastNode = function () {
        return this._list.tailNode();
    };
    return Queue;
}());
exports.default = Queue;
