export declare class NodeUnilateral<T> {
    value: T;
    next: NodeUnilateral<T> | null;
    constructor(value: T, next: NodeUnilateral<T> | null);
}
export default class SinglyLinkedList<T> {
    private _headNode;
    private _tailNode;
    private _length;
    /**
     * Creates a new instance of a singly-linked list.
     * @param values The inital values to populate list with. If omitted, list begins empty.
     */
    constructor(values?: T[]);
    /**
     * Inserts value at end of list. Time complexity = O(1).
     * @param value The value to append.
     * @returns The list instance - this.
     */
    append(value: T): this;
    /**
     * Inserts value at beginning of list. Time complexity = O(1).
     * @param value The value to prepend.
     * @returns The list instance - this.
     */
    prepend(value: T): this;
    /**
     * Inserts item at specified index. Time complexity = O(n) where n is number of items in list.
     * @param value The item to insert.
     * @param index The index at which to insert item.
     * @returns The list instance - this.
     */
    insert(value: T, index: number): this;
    /**
     * Removes an item at specified index. Throws error is list is empty. Time complexity = O(n) where n is number of items in list.
     * @param index The index of item to remove.
     * @returns The removed item.
     */
    remove(index: number): T | null;
    /**
     * Gets item at specified index. Time complexity = O(n) where n is number of items in list.
     * @param index The index of item to retrieve.
     * @returns The item at specified index.
     */
    get(index: number): T | null;
    _getNode(index: number): NodeUnilateral<T> | null;
    /**
     * @returns An array containing currently held items, ordered from head to tail. Time complexity = O(n) where n is number of items in list.
     */
    getItems(): T[];
    /**
     * Executes a callback function for each item in list. Time complexity = O(n) where n is number of items in list.
     * @param callback The function to execute for each item in list.
     * @returns The list instance - this.
     */
    forEach(callback: (item: T, index: number) => void): this;
    /**
     * Executes a callback function that replaces each item in list with result. Time complexity = O(n) where n is number of items in list.
     * @param callback The function to execute for each item in list.
     * @returns The list instance - this.
     */
    forEachMutate(callback: (item: T, index: number) => T): this;
    /**
     * Reverses items in list: head becomes tail and vice versa. Time complexity = O(n) where n is number of items in list.
     * @returns The list instance - this.
     */
    reverse(): this;
    /**
     * Removes all items from list. Time complexity = O(1).
     * @returns The list instance - this.
     */
    clear(): this;
    /**
     * Time complexity = O(1).
     * @returns True if list is empty, false otherwise.
     */
    isEmpty(): boolean;
    /**
     * Time complexity = O(1).
     * @returns The current number of items stored.
     */
    length(): number;
    /**
     * Time complexity = O(1).
     * @returns The current head value, or null if list is empty.
     */
    head(): T | null;
    /**
     * Time complexity = O(1).
     * @returns The current head node, or null if list is empty.
     */
    headNode(): NodeUnilateral<T> | null;
    /**
     * Time complexity = O(1).
     * @returns The current tail value, or null if list is empty.
     */
    tail(): T | null;
    /**
     * Time complexity = O(1).
     * @returns The current tail node, or null if list is empty.
     */
    tailNode(): NodeUnilateral<T> | null;
}
