export default class Stack<T> {
    private _values;
    private _top;
    private _bottom;
    /**
     * Creates a new instance of a stack (utilizes an array).
     * @param values The initial values to populate stack with. If omitted, stack begins empty.
     */
    constructor(values?: T[]);
    /**
     * Pushes a value onto top of stack. Time complexity = O(1).
     * @param value The value to append.
     * @returns The stack instance - this.
     */
    push(value: T): this;
    /**
     * Removes item at top of stack. Does nothing if stack is empty. Time complexity = O(1).
     * @returns The item at top of stack, or null if stack is empty.
     */
    pop(): T | null;
    /**
     * Time complexity = O(1).
     * @returns The item at top of stack, or null if stack is empty.
     */
    top(): T | null;
    /**
     * Time complexity = O(1).
     * @returns Item at bottom of stack, or null if stack is empty.
     */
    bottom(): T | null;
    /**
     * Time complexity = O(n) where n is number of items in stack.
     * @returns Array containing all currently held items, where first item is bottom and last item is top.
     */
    getItems(): T[];
    /**
     * Clears stack. Time complexity = O(1).
     */
    clear(): this;
    /**
     * Time complexity = O(1).
     * @returns True if stack is empty, false otherwise.
     */
    isEmpty(): boolean;
    /**
     * Time complexity = O(1).
     * @returns Number of items in stack.
     */
    length(): number;
}
