"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
var Stack = /** @class */ (function () {
    /**
     * Creates a new instance of a stack (utilizes an array).
     * @param values The initial values to populate stack with. If omitted, stack begins empty.
     */
    function Stack(values) {
        this._values = values !== undefined ? __spreadArray([], values, true) : [];
        if (values === undefined) {
            this._top = null;
        }
        else {
            this._top = values[values.length - 1] || null;
        }
        if (values === undefined) {
            this._bottom = null;
        }
        else {
            this._bottom = values[0] || null;
        }
    }
    /**
     * Pushes a value onto top of stack. Time complexity = O(1).
     * @param value The value to append.
     * @returns The stack instance - this.
     */
    Stack.prototype.push = function (value) {
        if (this.isEmpty()) {
            this._bottom = value;
        }
        this._values.push(value);
        this._top = value;
        return this;
    };
    /**
     * Removes item at top of stack. Does nothing if stack is empty. Time complexity = O(1).
     * @returns The item at top of stack, or null if stack is empty.
     */
    Stack.prototype.pop = function () {
        if (this.isEmpty()) {
            return null;
        }
        if (this._values.length === 1) {
            this._top = null;
            this._bottom = null;
        }
        else {
            this._top = this._values[this._values.length - 2];
        }
        return this._values.pop() || null;
    };
    /**
     * Time complexity = O(1).
     * @returns The item at top of stack, or null if stack is empty.
     */
    Stack.prototype.top = function () {
        return this._top || null;
    };
    /**
     * Time complexity = O(1).
     * @returns Item at bottom of stack, or null if stack is empty.
     */
    Stack.prototype.bottom = function () {
        return this._bottom || null;
    };
    /**
     * Time complexity = O(n) where n is number of items in stack.
     * @returns Array containing all currently held items, where first item is bottom and last item is top.
     */
    Stack.prototype.getItems = function () {
        return __spreadArray([], this._values, true);
    };
    /**
     * Clears stack. Time complexity = O(1).
     */
    Stack.prototype.clear = function () {
        this._values = [];
        this._top = null;
        this._bottom = null;
        return this;
    };
    /**
     * Time complexity = O(1).
     * @returns True if stack is empty, false otherwise.
     */
    Stack.prototype.isEmpty = function () {
        return this._values.length === 0;
    };
    /**
     * Time complexity = O(1).
     * @returns Number of items in stack.
     */
    Stack.prototype.length = function () {
        return this._values.length;
    };
    return Stack;
}());
exports.default = Stack;
